from zope.interface import Interface
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary

from bibliograph.core.encodings import _python_encodings

# XXX as long as we don't have a propper translation messagefactory
_ = unicode

###############################################################################

class IBibliographyExport(Interface):
    """ Marker interface for a container with
        exportable bibliography elements.
    """

    format = schema.Choice(
        title=_('Export format'),
        required=True,
        default='bibtex',
        # XXX make this dynamic using a factory utilizing 'enabled' &
        # 'available' flags          
        vocabulary=SimpleVocabulary.fromItems([
            (_("Bibtex"), 'bibtex'),
            (_("Endnote"), 'endnote'),
            (_("RIS"), 'ris'),
            (_("XML (MODS)"), 'xml'),
            (_("PDF"), 'pdf'),
        ])
        )

    output_encoding = schema.Choice(
        title=_('Encoding of output'),
        required=False,
        vocabulary=SimpleVocabulary.fromValues(_python_encodings)
        )

###############################################################################

class IBibliographicReference(Interface):
    """ An object is renderable as a bibliography """

    title = schema.TextLine(
        title=_('Title'),
        description=_('The title of the document'),
        required = True,
        )

    authors = schema.TextLine(
        title=_('Authors'),
        description=_('A formated string of the authors of the document'),
        required = True,
        )

    publication_type = schema.Choice(
        title=_('Publication type'),
        description=_('A publication type as found in the bibtex definition'),
        required=True,
        vocabulary=SimpleVocabulary.fromItems([
            (_("Article"), 'article'),
            (_("Book"), 'book'),
            (_("Booklet"), 'booklet'),
            (_("Conference"), 'conference'),
            (_("Inbook"), 'inbook'),
            (_("Incollection"), 'incollection'),
            (_("Inproceedings"), 'inproceenings'),
            (_("Manual"), 'manual'),
            (_("Masterthesis"), 'masterthesis'),
            (_("Misc"), 'misc'),
            (_("Phdthesis"), 'phdthesis'),
            (_("Proceedings"), 'proceedings'),
            (_("Techeport"), 'techreport'),
            (_("Unpublished"), 'unpublished'),
            ])
        )

    editor_flag = schema.Bool(
        title=_('Editor'),
        description=_('Marker for interpretation of author field as editor'),
        required = False,
        )

    publication_year = schema.Int(
        title=_('Year of publication'),
        required=True,
        )

    abstract = schema.Text(
        title=_('Abstract'),
        description=_('A short summary of the document'),
        required = False,
        )
    
    url = schema.TextLine(
        title=_('URL of the publication'),
        required=False,
        )

    subject = schema.List(
        title=_('Subject'),
        description=_('A list of tags(subjects) of the document'),
        required = True,
        )

    note = schema.Text(
        title=_('Note'),
        description=_('Some additional notes'),
        required = False,
        )

    annote = schema.Text(
        title=_('Annotation'),
        description=_('Some annotations'),
        required = False,
        )

    source_fields = schema.List(
        title=_('Source fields'),
        value_type=schema.Tuple(),
        required=False,
        )

    def getAuthors(*args, **kw):
        """ Get a list-like object containing the authors.
            The object must know about rendering a formatted list
            of authors by being called.
        """

# EOF



import unittest, doctest

from zope.testing import doctestunit
from zope.component import testing

class DummyEntry(object):

    def __init__(self, id=None):
        self.id = id

    def getId(self):
        return self.id

def test_suite():
    return unittest.TestSuite([

        # Unit tests for your API
        doctestunit.DocFileSuite(
            'README.txt',
            package='bibliograph.core',
            setUp=testing.setUp,
            tearDown=testing.tearDown),
            
        doctestunit.DocTestSuite(
            'bibliograph.core.utils',
            optionflags=doctest.ELLIPSIS,
            globs=dict(DummyEntry=DummyEntry))
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# -*- coding: utf-8 -*-
###############################################################################
# $Copy$
###############################################################################
""" Collection of python utility-methodes commonly used by other
    bibliograph packages.
    
$Id: utils.py 61541 2008-03-28 13:29:24Z tom_gross $
"""
__docformat__ = 'reStructuredText'
__author__  = 'Tom Gross <itconsense@gmail.com>'

# Python imports
import string
import re

# My imports ;-)
from bibliograph.core.encodings import _utf8enc2latex_mapping

###############################################################################

_default_encoding = 'utf-8'
_entity_mapping = {'&mdash;':'{---}',
                   '&ndash;':'{--}',
                   }

###############################################################################
# a poor-mans approach to fixing unicode issues :-(

def _encode(s, encoding=_default_encoding):
    ur""" Try to encode a string 
    
        >>> from bibliograph.core.utils import _encode
        
        ASCII is ASCII is ASCII ...
        >>> _encode(u'ascii', 'utf-8')
        'ascii'
        
        This is normal
        >>> _encode(u'öl', 'utf-8')
        '\xc3\xb6l'
        
        Don't fail on this ...
        >>> _encode('öl', 'utf-8')
        '\xc3\xb6l'
        
        Still throw an exception on unknown encodings        
        >>> _encode(u'öl', 'bogus')
        Traceback (most recent call last):
        ...
        LookupError: unknown encoding: bogus
        
    """
    try:
        return s.encode(encoding)
    except (TypeError, UnicodeDecodeError, ValueError):
        return s

def _decode(s, encoding=_default_encoding):
    ur""" Try to decode a string 

        >>> from bibliograph.core.utils import _decode
        
        ASCII is ASCII is ASCII ...
        >>> _decode('ascii', 'utf-8')
        u'ascii'
        
        This is normal
        >>> _decode('öl', 'utf-8')
        u'\xf6l'
        
        Don't fail on this ...
        >>> _decode(u'öl', 'utf-8')
        u'\xf6l'
        
        Still throw an exception on unknown encodings        
        >>> _decode('öl', 'bogus')
        Traceback (most recent call last):
        ...
        LookupError: unknown encoding: bogus
        
    """
    try:
        return unicode(s, encoding)
    except (TypeError, UnicodeDecodeError, ValueError):
        return s

###############################################################################

VALIDIDPAT = re.compile('[ "@\',\\#}{~%&$^]')

def _validKey(entry):
    """
    uses the Zope object id but
    removes characters not allowed in BibTeX keys

    >>> from bibliograph.core.utils import _validKey
    >>> _validKey(DummyEntry('Foo Bar'))
    'FooBar'

    >>> _validKey(DummyEntry('my@id'))
    'myid'

    """

    # be forward compatible to zope3 contained objects
    raw_id = getattr(entry, '__name__', '')
    if not raw_id:
        raw_id = entry.getId()

    # This substitution is based on the description of cite key restrictions at
    # http://bibdesk.sourceforge.net/manual/BibDesk%20Help_2.html
    return VALIDIDPAT.sub('', raw_id)

###############################################################################

def AuthorURLs(entry):
    """a string with all the known author's URLs;
    helper method for bibtex output"""
    a_URLs = ''
    for a in entry.getAuthors():
        url = a.get('homepage', ' ')
        a_URLs += "%s and " % url
    return a_URLs[:-5]

###############################################################################

def _braceUppercase(text):
    """ Convert uppercase letters to bibtex encoded uppercase
    
        >>> from bibliograph.core.utils import _braceUppercase
        >>> _braceUppercase('foo bar')
        'foo bar'
        
        >>> _braceUppercase('Foo Bar')
        '{F}oo {B}ar'
    """
    for uc in string.uppercase:
        text = text.replace(uc, r'{%s}' % uc)
    return text

###############################################################################

def _normalize(text, resolve_unicode=True):
    text.replace('\\', '\\\\')
    text = _resolveEntities(text)
    if resolve_unicode:
        text = _resolveUnicode(text)
    return _escapeSpecialCharacters(text)

###############################################################################

def _resolveEntities(text):
    for entity in _entity_mapping.keys():
        text = text.replace(entity, _entity_mapping[entity])
    return text

###############################################################################

def _resolveUnicode(text):
    for unichar in _utf8enc2latex_mapping.keys():
        text = _encode(_decode(text).replace(unichar,
                                             _utf8enc2latex_mapping[unichar]))
    return _encode(_decode(text).replace(r'$}{$',''))

###############################################################################

def _escapeSpecialCharacters(text):
    """
    latex escaping some (not all) special characters
    """
    text.replace('\\', '\\\\')
    escape = ['~', '#', '&', '%', '_']
    for c in escape:
        text = text.replace(c, '\\' + c )
    return text


###############################################################################

def title_or_id(context):
    """ Return the title or id.

        Works with Zope2 and Zope3
    """
    title = getattr(context, 'title', '')
    if not title:
        if hasattr(context, '__name__'):
            title = getattr(context, '__name__', '')
        elif hasattr(context, 'getId'):
            title = context.getId()
    return title

###############################################################################

def _convertToOutputEncoding(export_text,
                             input_encoding=None,
                             output_encoding='utf-8'):
    """ Convert the renderer's result to the output encoding
    """
    if input_encoding:
        return _encode(_decode(export_text, input_encoding), output_encoding)
    return _encode(_decode(export_text), output_encoding)

# EOF



# -*- coding: utf-8 -*-
"""
This module contains the core definitions of the bibliograph packages
"""
import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '0.1'

long_description = (
    read('README.txt')
    + '\n' +
    'Change history\n'
    '**************\n'
    + '\n' + 
    read('CHANGES.txt')
    + '\n' +
    'Detailed Documentation\n'
    '**********************\n'
    + '\n' +
    read('bibliograph', 'core', 'README.txt')
    + '\n' +
    'Contributors\n' 
    '************\n'
    + '\n' +
    read('CONTRIBUTORS.txt')
    + '\n' +
    'Download\n'
    '********\n'
    )

entry_points="""
# -*- Entry points: -*-
"""

setup(name='bibliograph.core',
      version=version,
      description="Core definitions of bibliograph packages",
      long_description=long_description,
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: Zope Public License",
        ],
      keywords='bibtex bibliography xml endnote ris bibutils',
      author='Tom Gross',
      author_email='itconsense@gmail.com',
      url='http://pypi.python.org/pypi/bibliograph.core',
      license='ZPL 2.1',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['bibliograph'],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools'],
      tests_require=['zope.testing'],
      test_suite = 'bibliograph.core.tests.test_suite',
      entry_points=entry_points,
      )
